/*
NAME
	make-word-table -
		Adjunct utility for the `wpoison' program.  Generates a
		randomized table of English words, using random selections
		from a standard UNIX(tm) dictionary file.

SYNOPSIS
	make-word-table

VERSION
	1.3

DESCRIPTION
	make-word-table scans through a dictionary file, pulling out every
	11th word.  If the word in question is 4 or more letters long, then
	it is added to the C language string array declaration which this
	program writes to its stdout file.

	The only use of this program is for gnerating new random word tables
	for the wpoison(1) program.

	This program is in kludge state right now and needs a lot of work.
	It *should* generate a randomized lists of words (of some specified
	size) from the dictionary file, but right now it isn't that smart
	and so it just grabs ever 11th word, and if it is long enough, it
	goes into the output words table.

NOTES
	When building this program, you should first check below to see that
	the pathname used as the definition of `words_pathname' constant is
	in fact the pathname for a dictionary file on your system.

COPYRIGHT
	Copyright (c) 1997; Ronald F. Guilmette; All rights Reserved.

	License to use and to redistribute this software for strictly non-
	commercial purposes is hereby granted (and encouraged) provided that
	the full verbatim text of this copyright notice, and the full verbatim
	text of the containing comment block are maintained in all source
	copies of this program at all times, and provided that licensee has
	read, agreed to, and accepted completely and without reservation
	the following LIMITATIONS OF WARRANTIES and LIMITATIONS OF LIABILITIES.

	LIMITATIONS OF WARRANTIES

	THE PROGRAM IS PROVIDED AS IS, AND LICENSOR DISCLAIMS ANY AND ALL
	REPRESENTATIONS OR WARRANTIES OF ANY KIND, WHETHER EXPRESS OR IMPLIED
	WITH RESPECT TO IT, INCLUDING (WITHOUT LIMITATION) ANY IMPLIED
	WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE
	OR NON-INFRINGEMENT.

	LIMITATIONS OF LIABILITIES

	LICENSOR SHALL HAVE NO LIABILITY TO LICENSEE OR TO ANY OTHER INDI-
	VIDUAL OR ENTITY CONNECTED WITH LICENSEE FOR ANY CLAIM, LOSS, OR
	DAMAGE OF ANY KIND OR NATURE WHATSOEVER ARISING OUT OF OR IN CON-
	NECTION WITH THE PERFORMANCE OF THIS AGREEMENT OR ARISING OUT OF
	OR IN CONNECTION WITH (I) THE DEFICIENCY OR INADEQUACY OF THE
	SOFTWARE FOR ANY PURPOSE, WHETHER KNOWN OR NOT KNOWN OR DISCLOSED
	TO LICENSEE; (II) THE USE OR PERFORMANCE OF THE SOFTWARE OR ANY
	FILES, OR COMPUTER SYSTEMS RELATED THERETO OR USED IN CONJUNCTION
	THEREWITH; (III) ANY INTERRUPTION OR LOSS OF SERVICE OR USE OF THE
	SOFTWARE, OR ANY FILES, DATA, OR OTHER COMPUTER SYSTEMS; (IV) ANY
	SOFTWARE OR SYSTEM FAILURE; OR (V) ANY LOSS OF PROFITS, SALES,
	BUSINESS, OR OTHER DIRECT, INDIRECT, INCIDENTAL, CONSEQUENTIAL, OR
	SPECIAL LOSS OR DAMAGE OF ANY KIND OR NATURE RESULTING FROM THE
	FORGOING.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>

extern long lrand48 (void);
extern void srand48 (long);

enum { min_word_len = 3 };

static char const *pname;

static char const *words_pathname = "/usr/dict/words";  /* default */

static char *words;
static char const **word_ptrs_base;
static char const **word_ptrs_ptr;
static char const **word_ptrs_limit;
static unsigned num_words;

static void
usage (void)
{
  fprintf (stderr, "%s: usage `%s'\n",
           pname, pname);
  exit (1);
}
 
static void *
xmalloc (register unsigned long const size)
{
  register void *const ret_val = malloc (size);
 
  if (ret_val == NULL)
    {
      fprintf (stderr, "%s: memory exhausted\n", pname);
      exit (1);
    }
  return ret_val;
}
 
static void *
xrealloc (register void *const p, register unsigned long const new_size)
{
  register void *const ret_val = realloc (p, new_size);
 
  if (ret_val == NULL)
    {
      fprintf (stderr, "%s: memory exhausted\n", pname);
      exit (1);
    }
  return ret_val;
}
 
static void
add_word (register char const *word_ptr)
{
  if (!word_ptrs_base)
    {
      word_ptrs_base = (char const **) xmalloc (4096 * sizeof (char *));
      word_ptrs_ptr = word_ptrs_base;
      word_ptrs_limit = word_ptrs_base + 4096;
    }
  else if (word_ptrs_ptr == word_ptrs_limit)
    {
      register size_t new_size = (word_ptrs_limit - word_ptrs_base) << 1U;
      register size_t used = (word_ptrs_ptr - word_ptrs_base);
 
      word_ptrs_base =
	(char const **) xrealloc (word_ptrs_base, new_size * sizeof (char *));
      word_ptrs_ptr = word_ptrs_base + used;
      word_ptrs_limit = word_ptrs_base + new_size;
    }
  *(word_ptrs_ptr++) = word_ptr;
}

int
main (int argc, char *argv[])
{
  struct stat stat_buf;
  register FILE *wfile;

  pname = strrchr (argv[0], '/');
  pname = pname ? pname+1 : argv[0];

  if (argc != 1)
    usage ();

  if ((wfile = fopen (words_pathname, "r")) == NULL)
    {
      fprintf (stderr, "%s: error opening dictionary file `%s': %s\n",
	       pname, words_pathname, strerror (errno));
      return 1;
    }

  if (stat (words_pathname, &stat_buf) != 0)
    {
      fprintf (stderr, "%s: error stat'ing dictionary file `%s': %s\n",
	       pname, words_pathname, strerror (errno));
      return 1;
    }

  words = (char *) xmalloc (stat_buf.st_size + 1);

  if (fread (words, 1, stat_buf.st_size, wfile) != stat_buf.st_size)
    {
      fprintf (stderr, "%s: error reading dictionary file `%s': %s\n",
	       pname, words_pathname, strerror (errno));
      return 1;
    }

  words[stat_buf.st_size] = '\0';

  if (fclose (wfile) != 0)
    {
      fprintf (stderr, "%s: error closing dictionary file `%s': %s\n",
	       pname, words_pathname, strerror (errno));
      return 1;
    }

  {
    register char *p;
    register char *q;
    register char const *limit;

    limit = &words[stat_buf.st_size];
    for (p = words; p < limit; p = q + 1)
      {
        register int has_apostrophy = 0;

        for (q = p; *q && *q != '\n'; q++)
	  if (*q == '\'')
	    has_apostrophy = 1;
        *q = '\0';
        if ((q - p) >= min_word_len && !has_apostrophy)
          add_word (p), num_words++;
      }
  }

  {
    register unsigned line_width = 0;
    register char const *const *p;

    puts ("static char const *const word_table[] = {");
    putchar (' '); line_width++;
    putchar (' '); line_width++;
    for (p = word_ptrs_base; p < word_ptrs_ptr; p += 11)
      {
	register unsigned wlen = strlen (*p);
	register unsigned item_len = 1 + 1 + wlen + 1 + 1;
	register unsigned new_line_width = line_width + item_len;

	if (new_line_width >= 72)
	  {
	    putchar ('\n');
	    line_width = 0;
	    putchar (' '); line_width++;
	    new_line_width = line_width + item_len;
	  }
	putchar (' ');
	putchar ('"');
	fputs (*p, stdout);
	putchar ('"');
	putchar (',');
	line_width = new_line_width;
      }

    printf ("\n  0\n};\nenum { num_words = (sizeof (word_table) / sizeof (char const *)) - 1 };\n");
  }

  return 0;
}
